@@ -24,9 +24,12 @@
 
 G_BEGIN_DECLS
 
-void    _ide_build_pipeline_cancel      (IdeBuildPipeline *self);
-VtePty *_ide_build_pipeline_get_pty     (IdeBuildPipeline *self);
-void    _ide_build_pipeline_set_message (IdeBuildPipeline *self,
-                                         const gchar      *message);
+void _ide_build_pipeline_cancel       (IdeBuildPipeline *self);
+void _ide_build_pipeline_set_message  (IdeBuildPipeline *self,
+                                       const gchar      *message);
+void _ide_build_pipeline_mark_broken  (IdeBuildPipeline *self);
+void _ide_build_pipeline_set_pty_size (IdeBuildPipeline *self,
+                                       guint             rows,
+                                       guint             columns);
 
 G_END_DECLS
