@@ -59,12 +59,12 @@ G_BEGIN_DECLS
 #include "buildsystem/ide-build-target.h"
 #include "buildsystem/ide-build-target-provider.h"
 #include "buildsystem/ide-compile-commands.h"
-#include "buildsystem/ide-configuration-manager.h"
-#include "buildsystem/ide-configuration.h"
-#include "buildsystem/ide-configuration-provider.h"
 #include "buildsystem/ide-dependency-updater.h"
 #include "buildsystem/ide-environment-variable.h"
 #include "buildsystem/ide-environment.h"
+#include "config/ide-configuration.h"
+#include "config/ide-configuration-manager.h"
+#include "config/ide-configuration-provider.h"
 #include "debugger/ide-debug-manager.h"
 #include "debugger/ide-debugger-breakpoint.h"
 #include "debugger/ide-debugger-breakpoints.h"
@@ -101,6 +101,8 @@ G_BEGIN_DECLS
 #include "editor/ide-editor-view.h"
 #include "files/ide-file-settings.h"
 #include "files/ide-file.h"
+#include "files/ide-indent-style.h"
+#include "files/ide-spaces-style.h"
 #include "genesis/ide-genesis-addin.h"
 #include "greeter/ide-greeter-section.h"
 #include "highlighting/ide-highlight-engine.h"
@@ -127,8 +129,6 @@ G_BEGIN_DECLS
 #include "preferences/ide-preferences-perspective.h"
 #include "preferences/ide-preferences-window.h"
 #include "projects/ide-project-edit.h"
-#include "projects/ide-project-file.h"
-#include "projects/ide-project-files.h"
 #include "projects/ide-project-info.h"
 #include "projects/ide-project-item.h"
 #include "projects/ide-project.h"
