@@ -125,13 +125,12 @@ ide_clang_service_build_index_visitor (CXCursor     cursor,
 
   if (style_name != NULL)
     {
-      CXString cxstr;
+      g_auto(CXString) cxstr = {0};
       const gchar *word;
 
       cxstr = clang_getCursorSpelling (cursor);
       word = clang_getCString (cxstr);
       ide_highlight_index_insert (request->index, word, (gpointer)style_name);
-      clang_disposeString (cxstr);
     }
 
   return CXChildVisit_Continue;
@@ -823,3 +822,24 @@ _ide_clang_dispose_string (CXString *str)
   if (str != NULL && str->data != NULL)
     clang_disposeString (*str);
 }
+
+void
+_ide_clang_dispose_diagnostic (CXDiagnostic *diag)
+{
+  if (diag != NULL)
+    clang_disposeDiagnostic (diag);
+}
+
+void
+_ide_clang_dispose_index (CXIndex *idx)
+{
+  if (idx != NULL && *idx != NULL)
+    clang_disposeIndex (*idx);
+}
+
+void
+_ide_clang_dispose_unit (CXTranslationUnit *unit)
+{
+  if (unit != NULL && *unit != NULL)
+    clang_disposeTranslationUnit (*unit);
+}
