@@ -20,8 +20,8 @@
 
 #include <glib/gi18n.h>
 
-#include "gbp-flatpak-configuration.h"
 #include "gbp-flatpak-download-stage.h"
+#include "gbp-flatpak-manifest.h"
 #include "gbp-flatpak-util.h"
 
 struct _GbpFlatpakDownloadStage
@@ -51,7 +51,6 @@ gbp_flatpak_download_stage_query (IdeBuildStage    *stage,
 {
   GbpFlatpakDownloadStage *self = (GbpFlatpakDownloadStage *)stage;
   IdeConfiguration *config;
-  GNetworkMonitor *monitor;
   g_autofree gchar *staging_dir = NULL;
   g_autofree gchar *manifest_path = NULL;
   g_autofree gchar *stop_at_option = NULL;
@@ -62,8 +61,21 @@ gbp_flatpak_download_stage_query (IdeBuildStage    *stage,
   g_assert (IDE_IS_BUILD_PIPELINE (pipeline));
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
+  /* Ignore downloads if there is no connection */
+  if (!g_network_monitor_get_network_available (g_network_monitor_get_default ()))
+    {
+      ide_build_stage_log (stage,
+                           IDE_BUILD_LOG_STDOUT,
+                           _("Network is not available, skipping downloads"),
+                           -1);
+      ide_build_stage_set_completed (stage, TRUE);
+      return;
+    }
+
   config = ide_build_pipeline_get_configuration (pipeline);
-  if (!GBP_IS_FLATPAK_CONFIGURATION (config))
+  g_assert (!config || IDE_IS_CONFIGURATION (config));
+
+  if (!GBP_IS_FLATPAK_MANIFEST (config))
     {
       ide_build_stage_set_completed (stage, TRUE);
       return;
@@ -73,31 +85,44 @@ gbp_flatpak_download_stage_query (IdeBuildStage    *stage,
     {
       g_autoptr(IdeSubprocessLauncher) launcher = NULL;
 
-      primary_module = gbp_flatpak_configuration_get_primary_module (GBP_FLATPAK_CONFIGURATION (config));
-      manifest_path = gbp_flatpak_configuration_get_manifest_path (GBP_FLATPAK_CONFIGURATION (config));
+      primary_module = gbp_flatpak_manifest_get_primary_module (GBP_FLATPAK_MANIFEST (config));
+      manifest_path = gbp_flatpak_manifest_get_path (GBP_FLATPAK_MANIFEST (config));
       staging_dir = gbp_flatpak_get_staging_dir (config);
       src_dir = ide_build_pipeline_get_srcdir (pipeline);
 
       launcher = ide_subprocess_launcher_new (G_SUBPROCESS_FLAGS_STDOUT_PIPE |
                                               G_SUBPROCESS_FLAGS_STDERR_PIPE);
 
-      ide_subprocess_launcher_set_run_on_host (launcher, TRUE);
-      ide_subprocess_launcher_set_clear_env (launcher, FALSE);
       ide_subprocess_launcher_set_cwd (launcher, src_dir);
+      ide_subprocess_launcher_set_run_on_host (launcher, FALSE);
+
+      if (ide_is_flatpak ())
+        {
+          g_autofree gchar *user_dir = NULL;
+
+          user_dir = g_build_filename (g_get_home_dir (), ".local", "share", "flatpak", NULL);
+          ide_subprocess_launcher_setenv (launcher, "FLATPAK_USER_DIR", user_dir, TRUE);
+          ide_subprocess_launcher_setenv (launcher, "XDG_RUNTIME_DIR", g_get_user_runtime_dir (), TRUE);
+        }
 
       ide_subprocess_launcher_push_argv (launcher, "flatpak-builder");
       ide_subprocess_launcher_push_argv (launcher, "--ccache");
       ide_subprocess_launcher_push_argv (launcher, "--force-clean");
+
       if (!dzl_str_empty0 (self->state_dir))
         {
           ide_subprocess_launcher_push_argv (launcher, "--state-dir");
           ide_subprocess_launcher_push_argv (launcher, self->state_dir);
         }
+
       ide_subprocess_launcher_push_argv (launcher, "--download-only");
+
       if (!self->force_update)
         ide_subprocess_launcher_push_argv (launcher, "--disable-updates");
+
       stop_at_option = g_strdup_printf ("--stop-at=%s", primary_module);
       ide_subprocess_launcher_push_argv (launcher, stop_at_option);
+
       ide_subprocess_launcher_push_argv (launcher, staging_dir);
       ide_subprocess_launcher_push_argv (launcher, manifest_path);
 
@@ -107,17 +132,6 @@ gbp_flatpak_download_stage_query (IdeBuildStage    *stage,
       self->invalid = FALSE;
       self->force_update = FALSE;
     }
-
-  /* Ignore downloads if there is no connection */
-  monitor = g_network_monitor_get_default ();
-  if (!g_network_monitor_get_network_available (monitor))
-    {
-      ide_build_stage_log (stage,
-                           IDE_BUILD_LOG_STDOUT,
-                           _("Network is not available, skipping downloads"),
-                           -1);
-      ide_build_stage_set_completed (stage, TRUE);
-    }
 }
 
 static void
@@ -162,8 +176,19 @@ gbp_flatpak_download_stage_class_init (GbpFlatpakDownloadStageClass *klass)
 
   stage_class->query = gbp_flatpak_download_stage_query;
 
+  /**
+   * GbpFlatpakDownloadStage:state-dir:
+   *
+   * The "state-dir" is the flatpak-builder state directory, to be used
+   * as a parameter to "flatpak-builder --state-dir".
+   *
+   * Since: 3.28
+   */
   properties [PROP_STATE_DIR] =
-    g_param_spec_string ("state-dir", NULL, NULL, NULL,
+    g_param_spec_string ("state-dir",
+                         "State Dir",
+                         "The flatpak-builder state directory",
+                         NULL,
                          G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, N_PROPS, properties);
