@@ -101,11 +101,11 @@ gb_project_tree_builder_add (GbProjectTreeBuilder *self,
   display_name = gb_project_file_get_display_name (item);
   icon_name = gb_project_file_get_icon_name (item);
 
+  is_dir = g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY;
+
   if (is_dir)
     expanded = "folder-open-symbolic";
 
-  is_dir = g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY;
-
   child = g_object_new (DZL_TYPE_TREE_NODE,
                         "children-possible", is_dir,
                         "reset-on-collapse", is_dir,
@@ -187,16 +187,16 @@ gb_project_tree_builder_remove (GbProjectTreeBuilder *self,
 }
 
 static void
-gb_project_tree_builder_changed (GbProjectTreeBuilder    *self,
-                                 GFile                   *file,
-                                 GFile                   *other_file,
-                                 GFileMonitorEvent        event,
-                                 DzlRecursiveFileMonitor *monitor)
+gb_project_tree_builder_changed (GbProjectTreeBuilder *self,
+                                 GFile                *file,
+                                 GFile                *other_file,
+                                 GFileMonitorEvent     event,
+                                 IdeVcsMonitor        *monitor)
 {
   g_assert (GB_PROJECT_TREE_BUILDER (self));
   g_assert (G_IS_FILE (file));
   g_assert (!other_file || G_IS_FILE (other_file));
-  g_assert (DZL_IS_RECURSIVE_FILE_MONITOR (monitor));
+  g_assert (IDE_IS_VCS_MONITOR (monitor));
 
   /*
    * We only handle CREATED in the normal signal connection, so that
@@ -221,16 +221,16 @@ gb_project_tree_builder_changed (GbProjectTreeBuilder    *self,
 }
 
 static void
-gb_project_tree_builder_changed_after (GbProjectTreeBuilder    *self,
-                                       GFile                   *file,
-                                       GFile                   *other_file,
-                                       GFileMonitorEvent        event,
-                                       DzlRecursiveFileMonitor *monitor)
+gb_project_tree_builder_changed_after (GbProjectTreeBuilder *self,
+                                       GFile                *file,
+                                       GFile                *other_file,
+                                       GFileMonitorEvent     event,
+                                       IdeVcsMonitor        *monitor)
 {
   g_assert (GB_PROJECT_TREE_BUILDER (self));
   g_assert (G_IS_FILE (file));
   g_assert (!other_file || G_IS_FILE (other_file));
-  g_assert (DZL_IS_RECURSIVE_FILE_MONITOR (monitor));
+  g_assert (IDE_IS_VCS_MONITOR (monitor));
 
   /*
    * We only handle DELETED in the after request so that we can ensure
@@ -247,6 +247,26 @@ gb_project_tree_builder_changed_after (GbProjectTreeBuilder    *self,
     }
 }
 
+static void
+gb_project_tree_builder_reloaded (GbProjectTreeBuilder *self,
+                                  IdeVcsMonitor        *monitor)
+{
+  DzlTree *tree;
+
+  g_assert (GB_IS_PROJECT_TREE_BUILDER (self));
+  g_assert (IDE_IS_VCS_MONITOR (monitor));
+
+  /*
+   * VCS reloaded, so queue a draw so we can update the colors on
+   * previously pixel-cached rows. We'll set the foreground when
+   * hooking into the pango attributes.
+   */
+
+  tree = dzl_tree_builder_get_tree (DZL_TREE_BUILDER (self));
+  if (tree != NULL)
+    gtk_widget_queue_draw (GTK_WIDGET (tree));
+}
+
 static void
 build_context (GbProjectTreeBuilder *self,
                DzlTreeNode          *node)
@@ -270,10 +290,16 @@ build_context (GbProjectTreeBuilder *self,
 
   if (!self->has_monitor)
     {
-      DzlRecursiveFileMonitor *monitor = ide_context_get_monitor (context);
+      IdeVcsMonitor *monitor = ide_context_get_monitor (context);
 
       self->has_monitor = TRUE;
 
+      g_signal_connect_object (monitor,
+                               "reloaded",
+                               G_CALLBACK (gb_project_tree_builder_reloaded),
+                               self,
+                               G_CONNECT_SWAPPED);
+
       g_signal_connect_object (monitor,
                                "changed",
                                G_CALLBACK (gb_project_tree_builder_changed),
