@@ -306,6 +306,8 @@ gbp_symbol_layout_stack_addin_update_tree (GbpSymbolLayoutStackAddin *self,
   self->cancellable = g_cancellable_new ();
 
   task = g_task_new (self, self->cancellable, NULL, NULL);
+  g_task_set_source_tag (task, gbp_symbol_layout_stack_addin_update_tree);
+  g_task_set_priority (task, G_PRIORITY_LOW);
 
   data = g_slice_new0 (SymbolResolverTaskData);
   data->resolvers = g_ptr_array_new_with_free_func (g_object_unref);
@@ -504,6 +506,9 @@ gbp_symbol_layout_stack_addin_set_view (IdeLayoutStackAddin *addin,
   g_assert (GBP_IS_SYMBOL_LAYOUT_STACK_ADDIN (self));
   g_assert (!view || IDE_IS_LAYOUT_VIEW (view));
 
+  /* First clear any old symbol tree */
+  gbp_symbol_menu_button_set_symbol_tree (self->button, NULL);
+
   if (IDE_IS_EDITOR_VIEW (view))
     buffer = ide_editor_view_get_buffer (IDE_EDITOR_VIEW (view));
 
